// functions for mod harbour
// GNRao

//----------------------------------------------------------------------------//

function MH_RecordSetToHTML( oRs, lHtmlPage )

   local cHTML    := ""
   local lBinary  := .f.
   local bm, n, t

   cHTML := [<table border="1" width="100%">]
   cHTML += "<tr>"
   for n := 0 to oRs:Fields:Count() - 1
      cHTML += ( "<td><b>" + oRs:Fields( n ):Name + "</b></td>" )
      if ( t := oRs:Fields( n ):Type ) == 204 .or. t == 205
         lBinary  := .t.
      endif
   next
   cHTML += "</tr>"

   if oRs:RecordCount() > 0
      bm    := oRs:Bookmark
      oRs:MoveFirst()
      if lBinary
         cHtml += MH_BinrayRsToHTML( oRs )
      else
         cHTML += "<tr><td>"
         cHTML += oRs:GetString( 2, oRs:RecordCount(), "</td><td>","</td></tr><tr><td>","&nbsp;" )
         cHTML += "</td></tr>"
      endif
      oRs:MoveFirst()
      oRs:BookMark   := bm
   endif
   cHTML += "</table>"

   if lHtmlPage == .t.
      cHTML := "<html>" + HB_EOL() + "<body>" + HB_EOL() + cHTML + HB_EOL() + "</body>" + HB_EOL() + "</html>"
   endif

return cHTML

//----------------------------------------------------------------------------//

function MH_BinrayRsToHTML( oRs )

   local cHtml := ""
   local n

   do while !oRs:Eof()
      cHtml    += "<tr>"
      for n := 0 to oRs:Fields:Count() - 1
         cHtml    += MH_ValToHTML( oRs:Fields( n ):Value )
      next
      cHtml    += "</tr>"
      oRs:MoveNext()
   enddo


return cHtml

//----------------------------------------------------------------------------//

function MH_DbfToHTML( cFieldList, lWebPage )

   local aHead := DBSTRUCT()

   AEval( aHead, { |a,i| aHead[ i ] := a[ 1 ] } )

return MH_ArrayToHTML( MH_DbfToArray( cFieldList ), aHead, lWebPage )

//----------------------------------------------------------------------------//

function MH_ArrayToHTML( aData, aHead, lWebPage )

   local cHtml := [<table border="1" width="100%">]

   cHtml    += MH_ValToHTML( aHead, "<b>" )
   cHtml    += MH_ValToHTML( aData )
   cHtml    += "</table>"

   if lWebPage == .t.
      cHTML := "<html>" + HB_EOL() + "<body>" + HB_EOL() + cHTML + HB_EOL() + "</body>" + HB_EOL() + "</html>"
   endif

return cHtml

//----------------------------------------------------------------------------//

function MH_DbfToArray( cFieldList, bFor, bWhile, nNext, nRec, lRest, aData )

   local aRet  := {}, nRecNo := RecNo(), bLine, cList

   // cFieldList : comma delimited list of fields. Eg: "First,City,Age"

   if ValType( aData ) == 'A'
      aRet     := aData
   endif

   if Empty( cFieldList ) .or. '*' $ cFieldList
      cList  := ""
      AEval( DbStruct(), { |a| cList += "," + a[ 1 ] } )
      cList  := Substr( cList, 2 )

      if Empty( cFieldList )
         cFieldList  := cList
      else
         cFieldList  := StrTran( cFieldList, '*', cList )
      endif

   endif

   bLine    := &( "{||{" + cFieldList + "}}" )
   DbEval( { || AAdd( aRet, Eval( bLine ) ) }, bFor, bWhile, nNext, nRec, lRest )
   DBGOTO( nRecNo )

return aRet

//----------------------------------------------------------------------------//

function MH_ImageToBase64( cImage )

   local cRet  := ""
   local cType

   if !Empty( cType := ImageType( cImage ) )
      cRet  := [<img src="data:image/] + ;
               cType + ";base64," + ;
               hb_Base64encode( cImage ) + [">]
   endif

return cRet

//----------------------------------------------------------------------------//

static function MH_ValToHTML( uVal, cTag )

   local cRet  := uVal
   local cType := ValType( uVal )

   do case
   case cType == "A"
      if Empty( uVal )
         return ""
      elseif HB_ISARRAY( uVal[ 1 ] )
         cRet  := ""
         AEval( uVal, { |u| cRet += MH_ValToHTML( u ) } )
         return cRet
      else
         cRet  := "<tr>"
         AEval( uVal, { |u| cRet += MH_ValToHTML( u, cTag ) } )
         return cRet + "</tr>"
      endif
   case cType $ "CM"
      if IsBinaryData( uVal, 15 )
         cRet  := MH_ImageToBase64( uVal )
      else
         cRet  := Trim( uVal )
      endif
   case cType == "D"
      cRet  := DTOC( uVal )
   case cType == "L"
      cRet  := If( uVal, "TRUE", "FALSE" )
   case cType == "N"
      cRet  := LTrim( Str( uVal ) )
   case cType == "T"
      cRet  := HB_TTOC( uVal )
   otherwise
      cRet  := ""
   endcase

   if cTag == nil
      cTag  := ""
   endif

return "<td>" + cTag + cRet + CloseTag( cTag ) + "</td>"

//----------------------------------------------------------------------------//

static function CloseTag( cTag )

   local nAt

   if Empty( cTag )
      cTag  := ""
   else
      cTag  := "</" + SubStr( cTag, 2 )
      if ( nAt := AT( " ", cTag ) ) > 0
         cTag  := Left( cTag, nAt - 1 )
      endif
   endif

return cTag

//----------------------------------------------------------------------------//

static function ImageType( cBuf )

   local cType, n, a
   local cPunct   := Chr( 9 ) + Chr( 10 ) + Chr( 12 ) + Chr( 13 ) + Chr( 26 ) + Chr( 141 )
   local lExact   := Set( _SET_EXACT, .f. )
   local lBinary  := .f.
   local aTypes   := { ;
         { "bmp",    "BM",                                  .t. }, ;
         { "png",    Chr( 0x89 ) + "PNG",                   .f. }, ;
         { "ico",    Chr( 0 ) + Chr( 0 ) + Chr( 1 ),        .f. }, ;
         { "jpeg",   Chr( 255 ) + Chr( 216 ) + Chr( 255 ),  .f. }, ;
         { "gif",    "GIF8",                                .t. }, ;
         { "tiff",   Chr(73) + Chr(73) + Chr(42),           .t. }, ;
         { "tiff",   Chr(77) + Chr(77) + Chr(42),           .t. }  }

   for each a in aTypes
      if cBuf = a[ 2 ]
         if a[ 3 ]
            if IsBinaryData( SubStr( cBuf, 3, 15 ) )
               cType := a[ 1 ]
            endif
         else
            cType    := a[ 1 ]
         endif
         EXIT
      endif
   next
   Set( _SET_EXACT, lExact )

return cType

//----------------------------------------------------------------------------//

static function IsBinaryData( cData, nLen )

   local c, n, lBinary  := .f.

   if nLen == nil
      nLen  := Min( 50, Len( cData ) )
   endif
   for n := 1 to nLen
      c  := SubStr( cData, n, 1 )
      if ( c < ' ' .and. !( c $ Chr( 9 ) + Chr( 10 ) + Chr( 12 ) + Chr( 13 ) + Chr(26) ) )
         lBinary  := .t.
         exit
      endif
   next

return lBinary

//----------------------------------------------------------------------------//

